/*------------------------------------------------------------------
chaotisches Verschluesselungsprogramm fuer Dateien

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.

Dieses Programm verschl&uuml;sselt Dateien sowohl mit XOR
als auch durch Vertauschung der Reihenfolge der Bytes

Verschl&uuml;sseln: cryptfil -e unverschluesselte_datei verschluesselte_datei password
Entschl&uuml;sseln: cryptfil -d verschluesselte_datei unverschluesselte_datei password
------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "include/ulicrypt.h"

#define _MY_LEN 4096

int help(void)
{
  printf("arguments:\n");
  printf("   -e encrypt; -d decrypt\n");
  printf("   source file\n");
  printf("   destination file\n");
  printf("   password\n");
  return 0;
}

int main(int _argc,char *_argv[])
{
  char encode=-1,*s;
  FILE *in=NULL,*out=NULL;
  __u8 *b;
  __u16 *w;
  __u32 len,countin=0,countout=0;
  random_seed seed;
  printf("\n\nxor encrypting/decrypting of files\n");
  printf("(c)2000 Ulrich Kritzner\n");
  printf("---\n");
  if (_argc!=5)
    return help();
  if (strcmp(_argv[1],"-e")==0)
    encode=1;
  if (strcmp(_argv[1],"-d")==0)
    encode=0;
  if (encode==-1)
    return help();
  in=fopen(_argv[2],"rb");                 /* open source file */
  if (in!=NULL)
    out=fopen(_argv[3],"wb");              /* open target file */
  if ((in==NULL)||(out==NULL)) {
    printf("mad mad error\n");
    if (in==NULL)
      printf("invalid source filename: %s\n",_argv[2]);
    else
      printf("invalid destination filename: %s\n",_argv[3]);
    if (in!=NULL)
      fclose(in);
    if (out!=NULL)
      fclose(out);
    printf("---\n");
    return help();
  }
  s=(char*)malloc(_MY_LEN);
  random_initseed(&seed,_argv[4],0,0);     /* create random numbers from password */
  b=crypt_generatebytes(_MY_LEN,&seed);    /* create xor-code from random numbers */
  w=crypt_generatekey(_MY_LEN,&seed);      /* create shake-code from random numbers */
  if (encode)
    printf("encrypt!\n");
  else
    printf("decrypt!\n");
  printf("---\n");
  printf("encoding... ");
  do {
    len=fread((void *)s,1,_MY_LEN,in);     /* read bytes from source file */
    countin+=len;
    if (encode)
      crypt_shake(len,_MY_LEN,s,w,1);      /* shake bytes for encrypt */
    crypt_xor(len,_MY_LEN,s,b);            /* xor-encrypt/decrypt */
    if (!encode)
      crypt_shake(len,_MY_LEN,s,w,0);      /* shake bytes for decrypt */
    countout+=fwrite((void *)s,1,len,out); /* write bytes to target file */
  } while ((len==_MY_LEN)&&(countin==countout));
  printf("%i of %i bytes\n",(int)countout,(int)countin);
  fclose(in);                              /* close source file */
  fclose(out);                             /* close target file */
  free(s);
  if (countin!=countout) {
    printf("error: destination file incomplete\n",_argv[2]);
    return 0;
  }
  return 1;
}
