#ifndef DROEPPEZ_CRYPT_256
#define DROEPPEZ_CRYPT_256

/*------------------------------------------------------------------
chaotische Verschluesselungsmaschine fuer Datensaetze
von exakt 256 Bytes Laenge

um einzelne Bytes bearbeiten zu koennen,
definiere man _CRYPT_ENABLE_CB

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.

Es wird sowohl mit XOR als auch Bytevertauschung verschluesselt
------------------------------------------------------------------*/

#define _CRYPT_BLOCKLEN 256
#define _CRYPT_BLOCKMASK 255

#include "u_random.h"

typedef struct{
          __u8 thexor[_CRYPT_BLOCKLEN],
               shake[_CRYPT_BLOCKLEN];
#ifdef _CRYPT_ENABLE_CB
          __u8 shakeback[_CRYPT_BLOCKLEN];
#endif
        } crypt_data256;
/* Datengrundlage fuer die Cryptografie
     thexor:    Bytefeld
     shake:     Mixerfeld 
     shakeback: Mixerfeld fuer rueckwaerts */

/* Deklarationen ----------------------------------- */

void crypt_initdata256(random_seed *seed,crypt_data256 *data,__u32 loops1,__u32 loops2);
/* initialisiert einen Codierungsschluessel
     seed:   Zufallssaat
     data:   Codierungsschluessel
     loops1: Anzahl der Einschwingzyklen
     loops2: Anzahl der Zwischenzyklen */
     
void crypt_block256(crypt_data256 *data,char *inblock,char *outblock,char encrypt);
/* Codiert einen 1024-Byte-Block
     data:     Codierungsschluessel
     inblock:  Quellblock
     outblock: Zielblock
     encrypt:  Verschluesseln:1 Entschluesseln:0 */
     
void crypt_byte256(crypt_data256 *data,__u8 *n,char *c,char encrypt);
/* Codiert ein byte und seine Nummer
     data:    Codierungsschluessel
     n:       Nummer des Bytes im Block
     c:       Byte
     encrypt: Verschluesseln:1 Entschluesseln:0 */

/* Prozeduren -------------------------------------- */

void crypt_initdata256(random_seed *seed,crypt_data256 *data,__u32 loops1,__u32 loops2)
{
  __u16 i,b,nr,buf[_CRYPT_BLOCKLEN];
  random_loops(seed,loops1);
  for (i=0;i<_CRYPT_BLOCKLEN;i++)
    data->thexor[i]=(__u8)((__u16)random_random(seed)&0xFF);
  random_loops(seed,loops2);
  for (i=0;i<_CRYPT_BLOCKLEN;i++)
    buf[i]=random_random(seed)&_CRYPT_BLOCKMASK;
  for (i=0;i<_CRYPT_BLOCKLEN;i++)
    data->shake[i]=i;
  for (i=0;i<_CRYPT_BLOCKLEN;i++)
  {
    nr=buf[i];
    b=data->shake[i];
    data->shake[i]=data->shake[nr];
    data->shake[nr]=b;
  }
#ifdef _CRYPT_ENABLE_CB
  for (i=0;i<_CRYPT_BLOCKLEN;i++)
    data->shakeback[data->shake[i]]=i;
#endif
}

void crypt_block256(crypt_data256 *data,char *inblock,char *outblock,char encrypt)
{
  __u16 i;
  if (encrypt)
    for (i=0;i<_CRYPT_BLOCKLEN;i++)
      outblock[data->shake[i]]=inblock[i]^data->thexor[i];
  else
    for (i=0;i<_CRYPT_BLOCKLEN;i++)
      outblock[i]=inblock[data->shake[i]]^data->thexor[i];
}

#ifdef _CRYPT_ENABLE_CB
void crypt_byte256(crypt_data256 *data,__u8 *n,char *c,char encrypt)
{
  if (encrypt) {
    *c=*c^data->thexor[*n];
    *n=data->shake[*n];
  } else {
    *n=data->shakeback[*n];
    *c=*c^data->thexor[*n];
  }
}
#endif

#endif
