#ifndef DROEPPEZ_CRYPT_1024
#define DROEPPEZ_CRYPT_1024

/*------------------------------------------------------------------
chaotische Verschluesselungsmaschine fuer Datensaetze
von exakt 1024 Bytes Laenge

um einzelne Bytes bearbeiten zu koennen,
definiere man _CRYPT_ENABLE_CB

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.

Es wird sowohl mit XOR als auch
Bytevertauschung innerhalb von 256-Byte-Bloecken
und Blockvertauschung verschluesselt
------------------------------------------------------------------*/

#define _CRYPT_BLOCKCOUNT 4

#include "crypt256.h"

typedef struct{
          crypt_data256 data[_CRYPT_BLOCKCOUNT];
          __u8 shake[_CRYPT_BLOCKCOUNT];
#ifdef _CRYPT_ENABLE_CB
          __u8 shakeback[_CRYPT_BLOCKCOUNT];
#endif
        } crypt_data1024;
/* Datengrundlage fuer die Cryptografie,
   zusammengesetzt aus vier 256-er-Feldern */

/* Deklarationen ----------------------------------- */

void crypt_initdata1024(random_seed *seed,crypt_data1024 *data,__u32 loops1,__u32 loops2,__u32 loops3);
/* initialisiert einen Codierungsschluessel
     seed:   Zufallssaat
     data:   Codierungsschluessel
     loops1: Anzahl der Anfangszyklen
     loops2: Anzahl der Einschwingzyklen
     loops3: Anzahl der Zwischenzyklen */
     
void crypt_block1024(crypt_data1024 *data,char *inblock,char *outblock,char encrypt);
/* Codiert einen 1024-Byte-Block
     data:     Codierungsschluessel
     inblock:  Quellblock
     outblock: Zielblock
     encrypt:  Verschluesseln:1 Entschluesseln:0 */
     
void crypt_byte1024(crypt_data1024 *data,__u16 *n,char *c,char encrypt);
/* Codiert ein byte und seine Nummer
     data:    Codierungsschluessel
     n:       Nummer des Bytes im Block
     c:       Byte
     encrypt: Verschluesseln:1 Entschluesseln:0 */

/* Prozeduren -------------------------------------- */

void crypt_initdata1024(random_seed *seed,crypt_data1024 *data,__u32 loops1,__u32 loops2,__u32 loops3)
{
  __u8 i,s;
  __u16 n[_CRYPT_BLOCKCOUNT];
  random_loops(seed,loops1);
  for (i=0;i<_CRYPT_BLOCKCOUNT;i++)
    crypt_initdata256(seed,&data->data[i],loops2,loops3);
  random_loops(seed,loops1);
  for (i=0;i<_CRYPT_BLOCKCOUNT;i++)
  {
    data->shake[i]=i;
    n[i]=random_random(seed)&3;
  }
  for (i=0;i<_CRYPT_BLOCKCOUNT;i++)
  {
    s=data->shake[i];
    data->shake[i]=data->shake[n[i]];
    data->shake[n[i]]=s;
  }
#ifdef _CRYPT_ENABLE_CB
  for (i=0;i<_CRYPT_BLOCKCOUNT;i++)
    data->shakeback[data->shake[i]]=i;
#endif
}

void crypt_block1024(crypt_data1024 *data,char *inblock,char *outblock,char encrypt)
{
  __u8 i;
  if (encrypt)
    for (i=0;i<_CRYPT_BLOCKCOUNT;i++)
      crypt_block256(&data->data[i],inblock+i*_CRYPT_BLOCKLEN,outblock+data->shake[i]*_CRYPT_BLOCKLEN,encrypt);
  else
    for (i=0;i<_CRYPT_BLOCKCOUNT;i++)
      crypt_block256(&data->data[i],inblock+data->shake[i]*_CRYPT_BLOCKLEN,outblock+i*_CRYPT_BLOCKLEN,encrypt);
}

#ifdef _CRYPT_ENABLE_CB
void crypt_byte1024(crypt_data1024 *data,__u16 *n,char *c,char encrypt)
{
  __u8 b,i;
  i=(__u8)((__u16)*n>>8);
  b=(__u8)((__u16)*n&0xFF);
  if (encrypt) {
    crypt_byte256(&data->data[i],&b,c,encrypt);
    i=data->shake[i];
  } else {
    i=data->shakeback[i];
    crypt_byte256(&data->data[i],&b,c,encrypt);
  }
  *n=((__u16)i<<8)+b;
}
#endif

#endif
