#ifndef DROEPPEZ_CRYPT_BLK
#define DROEPPEZ_CRYPT_BLK

/*------------------------------------------------------------------
chaotische Verschluesselungsmaschine fuer Datensaetze
von bis zu 65536 Bytes Laenge

um einzelne Bytes bearbeiten zu koennen,
definiere man _CRYPT_ENABLE_CB

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.

Die Schluessellaenge muss der Laenge der Datensaetze entsprechen

Es wird sowohl mit XOR als auch Bytevertauschung verschluesselt
------------------------------------------------------------------*/

#include <malloc.h>

#ifndef _CRYPT_MAXLEN
#define _CRYPT_MAXLEN 65536
#endif

#include "u_random.h"

typedef struct{
          __u32 len;
          __u8  *thexor;
          __u16 *shake;
#ifdef _CRYPT_ENABLE_CB
          __u16 *shakeback;
#endif
        } crypt_data_blk;
/* Datengrundlage fuer die Cryptografie
     len:       Blocklaenge
     thexor:    Bytefeld
     shake:     Mixerfeld 
     shakeback: Mixerfeld fuer rueckwaerts */

/* Deklarationen ----------------------------------- */

crypt_data_blk *crypt_createdata_blk(random_seed *seed,__u32 len,__u32 loops1,__u32 loops2);
/* erzeugt einen Codierungsschluessel
     seed:   Zufallssaat
     len:    Codelaenge
     loops1: Anzahl der Einschwingzyklen
     loops2: Anzahl der Zwischenzyklen
     Rueckgabewert: Codierungsschluessel */

crypt_data_blk *crypt_freedata_blk(crypt_data_blk *data);
/* loescht einen Codierungsschluessel */

void crypt_block_blk(crypt_data_blk *data,char *inblock,char *outblock,char encrypt);
/* Codiert einen 1024-Byte-Block
     data:     Codierungsschluessel
     inblock:  Quellblock
     outblock: Zielblock
     encrypt:  Verschluesseln:1 Entschluesseln:0 */

void crypt_byte_blk(crypt_data_blk *data,__u16 *n,char *c,char encrypt);
/* Codiert ein byte und seine Nummer
     data:    Codierungsschluessel
     n:       Nummer des Bytes im Block
     c:       Byte
     encrypt: Verschluesseln:1 Entschluesseln:0 */

/* Prozeduren -------------------------------------- */

crypt_data_blk *crypt_createdata_blk(random_seed *seed,__u32 len,__u32 loops1,__u32 loops2)
{
  __u32 i,b;
  __u16 nr,*buf;
  crypt_data_blk *data;
  data=(crypt_data_blk *)malloc(sizeof(crypt_data_blk));
  if (data==NULL)
    return NULL;
  if (len>_CRYPT_MAXLEN)
    len=_CRYPT_MAXLEN;
  data->len=len;
  buf=(__u16*)malloc(sizeof(__u16)*len);
  data->thexor=(__u8*)malloc(sizeof(__u8)*len);
  data->shake=(__u16*)malloc(sizeof(__u16)*len);
#ifdef _CRYPT_ENABLE_CB
  data->shakeback=(__u16*)malloc(sizeof(__u16)*len);
#endif
  if ((data->thexor==NULL)||(data->shake==NULL))
    return crypt_freedata_blk(data);
#ifdef _CRYPT_ENABLE_CB
  if (data->shakeback==NULL)
    return crypt_freedata_blk(data);
#endif
  random_loops(seed,loops1);
  for (i=0;i<data->len;i++)
    data->thexor[i]=(__u8)((__u16)random_random(seed)&0xFF);
  random_loops(seed,loops2);
  for (i=0;i<data->len;i++)
    buf[i]=random_random(seed)%data->len;
  for (i=0;i<data->len;i++)
    data->shake[i]=i;
  for (i=0;i<data->len;i++)
  {
    nr=buf[i];
    b=data->shake[i];
    data->shake[i]=data->shake[nr];
    data->shake[nr]=b;
  }
#ifdef _CRYPT_ENABLE_CB
  for (i=0;i<data->len;i++)
    data->shakeback[data->shake[i]]=i;
#endif
  free(buf);
  return data;
}

crypt_data_blk *crypt_freedata_blk(crypt_data_blk *data)
{
  if (data==NULL)
    return NULL;
  if (data->thexor!=NULL)
    free(data->thexor);
  if (data->shake!=NULL)
    free(data->shake);
#ifdef _CRYPT_ENABLE_CB
  if (data->shakeback!=NULL)
    free(data->shakeback);
#endif
  free(data);
  return (crypt_data_blk *) NULL;
}

void crypt_block_blk(crypt_data_blk *data,char *inblock,char *outblock,char encrypt)
{
  __u32 i;
  if (encrypt)
    for (i=0;i<data->len;i++)
      outblock[data->shake[i]]=inblock[i]^data->thexor[i];
  else
    for (i=0;i<data->len;i++)
      outblock[i]=inblock[data->shake[i]]^data->thexor[i];
}

#ifdef _CRYPT_ENABLE_CB
void crypt_byte_blk(crypt_data_blk *data,__u16 *n,char *c,char encrypt)
{
  if (encrypt) {
    *c=*c^data->thexor[*n];
    *n=data->shake[*n];
  } else {
    *n=data->shakeback[*n];
    *c=*c^data->thexor[*n];
  }
}
#endif

#endif
