#ifndef DROEPPEZ_CRYPT_L_1024
#define DROEPPEZ_CRYPT_L_1024

/*------------------------------------------------------------------
chaotische Verschluesselungsmaschine fuer Datensaetze
von exakt 1024 Bytes Laenge

um einzelne Bytes bearbeiten zu koennen,
definiere man _CRYPT_ENABLE_CB

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.

Es wird sowohl mit XOR als auch 4-Byte-Vertauschung verschluesselt
------------------------------------------------------------------*/

#define _CRYPT_BLOCKLEN_L 256
#define _CRYPT_BLOCKMASK_L 255
#define _CRYPT_BLOCKLEN_X 1024

#include "u_random.h"

typedef struct{
          __u32 thexor[_CRYPT_BLOCKLEN_L];
          __u8 shake[_CRYPT_BLOCKLEN_L];
#ifdef _CRYPT_ENABLE_CB
          __u8 shakeback[_CRYPT_BLOCKLEN_L];
#endif
        } crypt_datal1024;
/* Datengrundlage fuer die Cryptografie
     thexor:    Bytefeld
     shake:     Mixerfeld 
     shakeback: Mixerfeld fuer rueckwaerts */

/* Deklarationen ----------------------------------- */

void crypt_initdatal1024(random_seed *seed,crypt_datal1024 *data,__u32 loops1,__u32 loops2);
/* initialisiert einen Codierungsschluessel
     seed:   Zufallssaat
     data:   Codierungsschluessel
     loops1: Anzahl der Einschwingzyklen
     loops2: Anzahl der Zwischenzyklen */
     
void crypt_blockl1024(crypt_datal1024 *data,char *inb,char *outb,char encrypt);
/* Codiert einen 1024-Byte-Block
     data:     Codierungsschluessel
     inblock:  Quellblock
     outblock: Zielblock
     encrypt:  Verschluesseln:1 Entschluesseln:0 */
     
void crypt_bytel1024(crypt_datal1024 *data,__u16 *n,char *c,char encrypt);
/* Codiert ein byte und seine Nummer
     data:    Codierungsschluessel
     n:       Nummer des Bytes im Block
     c:       Byte
     encrypt: Verschluesseln:1 Entschluesseln:0 */

/* Prozeduren -------------------------------------- */

void crypt_initdatal1024(random_seed *seed,crypt_datal1024 *data,__u32 loops1,__u32 loops2)
{
  __u16 i,b,nr,buf[_CRYPT_BLOCKLEN_L];
  __u8 *thexor;
  thexor=(__u8*)&data->thexor[0];
  random_loops(seed,loops1);
  for (i=0;i<_CRYPT_BLOCKLEN_X;i++)
    thexor[i]=(__u8)((__u16)random_random(seed)&0xFF);
  random_loops(seed,loops2);
  for (i=0;i<_CRYPT_BLOCKLEN_L;i++)
    buf[i]=random_random(seed)&_CRYPT_BLOCKMASK_L;
  for (i=0;i<_CRYPT_BLOCKLEN_L;i++)
    data->shake[i]=i;
  for (i=0;i<_CRYPT_BLOCKLEN_L;i++)
  {
    nr=buf[i];
    b=data->shake[i];
    data->shake[i]=data->shake[nr];
    data->shake[nr]=b;
  }
#ifdef _CRYPT_ENABLE_CB
  for (i=0;i<_CRYPT_BLOCKLEN_L;i++)
    data->shakeback[data->shake[i]]=i;
#endif
}

void crypt_blockl1024(crypt_datal1024 *data,char *inb,char *outb,char encrypt)
{
  __u16 i;
  __u32 *inblock,*outblock;
  inblock=(__u32*) inb;
  outblock=(__u32*) outb;
  if (encrypt)
    for (i=0;i<_CRYPT_BLOCKLEN_L;i++)
      outblock[data->shake[i]]=inblock[i]^data->thexor[i];
  else
    for (i=0;i<_CRYPT_BLOCKLEN_L;i++)
      outblock[i]=inblock[data->shake[i]]^data->thexor[i];
}

#ifdef _CRYPT_ENABLE_CB
void crypt_bytel1024(crypt_datal1024 *data,__u16 *n,char *c,char encrypt)
{
  __u8 nb,*thexor;
  thexor=(__u8*)&data->thexor[0];
  nb=*n&3;
  if (encrypt) {
    *c=*c^thexor[*n];
    *n=data->shake[*n>>2];
    *n=(*n<<2)+nb;
  } else {
    *n=data->shakeback[*n>>2];
    *n=(*n<<2)+nb;
    *c=*c^thexor[*n];
  }
}
#endif

#endif
