#ifndef DROEPPEZ_NICE_TEXT
#define DROEPPEZ_NICE_TEXT

/*------------------------------------------------------------------
Schluesselt beliebige Zeichenketten in menschenlesbaren Text um,
auch solche Zeichenketten,
die haessliche Zeichen und auch Nullen enthalten.
Das gleiche natuerlich auch wieder rueckwaerts.

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.
------------------------------------------------------------------*/

#include "u_types.h"

#ifndef NICE_SEP_1
#define NICE_SEP_1 '-'
#endif
#ifndef NICE_SEP_2
#define NICE_SEP_2 '.'
#endif
#ifndef UGLY_ERROR_LEN
#define UGLY_ERROR_LEN "error: invalid length of string"
#endif
#ifndef UGLY_ERROR_CHECK
#define UGLY_ERROR_CHECK "error: invalid checkbyte"
#endif
#ifndef UGLY_ERROR_CHAR
#define UGLY_ERROR_CHAR "error: invalid character at position %i"
#endif

/* Deklarationen ----------------------------------- */

void ugly_to_nice(__u8 *nice,__u8 *ugly,__u32 uglylen);
/* Wandelt eine haessliche Zeichenkette der Laenge uglylen
   in eine menschenlesbare Zeichenkette um
     nice: Ausgabe lesbare Zeichenkette
           sollte Speicher fuer die dreifache Laenge von ugly bieten
     ugly: Eingabe haessliche Zeichenkette
     uglylen: Laenge der haesslichen Zeichenkette */
char nice_to_ugly(__u8 *ugly,__u8 *nice,__s32 *erg);
/* Wandelt eine lesbare Zeichenkette in eine haessliche Zeichenkette um
     ugly: Ausgabe haessliche Zeichenkette
           sollte auch genuegend Speicherplatz fuer eine Fehlermeldung anbieten,
           die im Fehlerfalle ausgegeben wird
           (siehe Deklaration oben)
     nice: Eingabe lesbare Zeichenkette
     erg:  eventueller Fehlercode
       -2  : Checkbyte der lesbaren Kette stimmt nicht ueberein 
       -1  : Laenge der lesbaren Kette stimmt nicht 
        0  : alles OK
       1...: Nummer des ersten ungueltigen Zeichens (beginnend mit 1 anstatt 0)
     Rueckgabewert: 1 bei Erfolg, sonst 0 */

/* Prozeduren -------------------------------------- */

void ugly_to_nice(__u8 *nice,__u8 *ugly,__u32 uglylen)
{
  __u32 i,j;
  __u8 s[3],c;
  j=0;
  c=0;
  for (i=0;i<uglylen;i++) {
    c=c^ugly[i];
    s[0]=ugly[i]%26+'a';
    s[1]=ugly[i]/26+'0';
    s[2]=NICE_SEP_1;
    if ((i%4)==3)
      s[2]=NICE_SEP_2;
    nice[j]=s[0];
    nice[j+1]=s[1];
    nice[j+2]=s[2];
    j+=3;
  }
  s[0]=c%26+'a';
  s[1]=c/26+'0';
  nice[j]=s[0];
  nice[j+1]=s[1];
  j+=2;
  nice[j]=0;
}

__s32 my_nice_to_ugly(__u8 *ugly,__u8 *nice)
{
  __u32 i,j,uglylen;
  __u8 c,cs,s[3];
  ugly[0]=0;
  for (j=0;nice[j]!=0;j++) {
    i=j/3;
    switch (j%3) {
      case 0: {
        if ((nice[j]<'a')||(nice[j]>'z'))
          return j+1;
      } break;
      case 1: {
        if ((nice[j]<'0')||(nice[j]>'9'))
          return j+1;
      } break;
      case 2: {
        if ((i%4)==3) {
          if (nice[j]!=NICE_SEP_2)
            return j+1;
        } else {
          if (nice[j]!=NICE_SEP_1)
            return j+1;
        }
      } break;
    }
  }
  uglylen=i+1;
  if (j!=(uglylen*3)-1)
    return -1;
  if (uglylen<1)
    return -1;
  uglylen-=1;
  c=0;
  for (i=0;i<uglylen;i++) {
    j=i*3;
    ugly[i]=(nice[j]-'a')+((nice[j+1]-'0')*26);
    c=c^ugly[i];
  }
  j=i*3;
  ugly[i]=0;
  cs=(nice[j]-'a')+((nice[j+1]-'0')*26);
  if (cs!=c) {
    for (i=0;i<uglylen;i++)
      ugly[i]=0;
    return -2;
  }
  return 0;
}

char nice_to_ugly(__u8 *ugly,__u8 *nice,__s32 *erg)
{
  *erg=my_nice_to_ugly(ugly,nice);
  switch (*erg) {
    case 0: return 1; break;
    case -1: sprintf(ugly,UGLY_ERROR_LEN); break;
    case -2: sprintf(ugly,UGLY_ERROR_CHECK); break;
    default: sprintf(ugly,UGLY_ERROR_CHAR,(int)*erg); break;
  }
  return 0;
}

#endif
