#ifndef DROEPPEZ_U_PARITY
#define DROEPPEZ_U_PARITY
#include "u_types.h"

/*------------------------------------------------------------------
Paritaetsgenerierung und Pruefung

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.
------------------------------------------------------------------*/

__u32 parity_generate(__u8 *data,__u32 h,__u32 w);
/* generiert ein rechteckiges Paritaetsmuster
     data: Datenbasis
     h,w:  Hoehe und Breite des Rechtecks
     Rueckgabewert: neue Groesse des Datenpakets
   ---> Hinweis: h und w geben die Groesse der Datenbasis an.
                 Die Groesse von data muss vor dem Versehen mit Paritaeten
                 h*w Bytes gross sein, nach dem Generieren der Paritaeten
                 muss es dagegen gross genug sein, um (h+1)*(w+1) Bytes zu fassen. */
__u8 parity_check(__u8 *data,__u32 h,__u32 w);
/* prueft die Paritaeten
     data: Datenbasis einschliesslich Paritaetsmuster
     h,w:  Hoehe und Breite des Rechtecks
     Rueckgabewert: 1, wenn Paritaeten zulaessig
  ---> Hinweis: Das Paritaetenmuster der Datenbasis wird bei dem Vorgang veraendert
                und ist somit nachher nicht mehr verfuegbaer */

__u32 parity_generate(__u8 *data,__u32 h,__u32 w)
{
  __u32 x,y,size;
  __u8 *pvert,*phorz,*parity;
  size=h*w;
  pvert=&(data[size]);
  phorz=&(data[size+h]);
  parity=&(data[size+h+w]);
  for (x=0;x<w;x++)
    phorz[x]=0;
  for (y=0;y<h;y++)
    pvert[y]=0;
  *parity=0;
  for (x=0;x<w;x++)
    for (y=0;y<h;y++)
      {
        phorz[x]=phorz[x]^data[x+y*w];
        pvert[y]=pvert[y]^data[x+y*w];
      }
  for (y=0;y<h;y++)
    *parity=*parity^pvert[y];
  return (h+1)*(w+1);
}

__u8 parity_check(__u8 *data,__u32 h,__u32 w)
{
  __u32 x,y,size;
  __u8 *pvert,*phorz,*parity,result;
  result=1;
  size=h*w;
  pvert=&(data[size]);
  phorz=&(data[size+h]);
  parity=&(data[size+h+w]);
  for (y=0;y<h;y++)
    *parity=*parity^pvert[y];
  for (x=0;x<w;x++)
    for (y=0;y<h;y++)
      {
        phorz[x]=phorz[x]^data[x+y*w];
        pvert[y]=pvert[y]^data[x+y*w];
      }
  for (x=0;x<w;x++)
    if (phorz[x]!=0)
      result=0;
  for (y=0;y<h;y++)
    if (pvert[y]!=0)
      result=0;
  if (*parity!=0)
    result=0;
  return result;
}

#endif
