#ifndef DROEPPEZ_CRYPT
#define DROEPPEZ_CRYPT

/*------------------------------------------------------------------
chaotische Verschluesselungsmaschine fuer Datensaetze
von bis zu 65536 Bytes Laenge

(c) 2002 Ulrich Kritzner
Die Verbreitung dieser Datei unterliegt der GNU General Public License.

Copyright 2002 by Ulrich Kritzner.
Redistribution of this file is permitted under the GNU General Public License.

Die Schluessellaenge kann unabhaengig
von der Laenge der zu verschluesselnden Datensaetze gewaehlt werden.
Auch die Datensaetze koennen untereinander unterschliedliche
Laengen haben.

Es wird sowohl XOR als auch Bytevertauschung angeboten
------------------------------------------------------------------*/

#include <malloc.h>
#include "u_random.h"

/* Deklarationen ----------------------------------- */

__u16 *crypt_generatekey(__u32 n,random_seed *seed);
/* erstellt eine Folge von 16-Bit-Zahlen
     n:    Codelaenge
     seed: Zufallszahlensaat
     Rueckgabe: Folge von 16-Bit-Zahlen */

__u16 *crypt_freekey(__u16 *key);
/* loescht eine Folge von 16-Bit-Zahlen aus dem Speicher */

__u8 *crypt_generatebytes(__u32 n,random_seed *seed);
/* erstellt eine Folge von Bytes
     n:    Codelaenge
     seed: Zufallszahlensaat
     Rueckgabe: Folge von Bytes */

__u8 *crypt_convertbytes(__u32 n,__u16 *key);
/* erstellt eine Folge von Bytes aus einem Schluessel von 16-Bit-Zahlen
     n:   Codelaenge (Anzahl der 16-Bit-Zahlen)
     key: Folge von 16-Bit-Zahlen
     Rueckgabe: Folge von Bytes */

char crypt_xor(__u32 len,__u32 n,__u8 *data,__u8 *code);
/* verknuepft data mit code ueber xor
     len:  Laenge der Datenbasis in Bytes
     n:    Codelaenge
     data: zu ver-/entschluesselnde Daten
     code: Byteschluessel */

char crypt_shake(__u32 len,__u32 n,__u8 *data,__u16 *code,char encode);
/* Schuettelt data anhand von code durcheinander
     len:    Laenge der Datenbasis in Bytes
     n:      Codelaenge
     data:   zu ver-/entschluesselnde Daten
     code:   Byteschluessel
     encode: 1 bei Codierung, 0 bei Decodierung */

__u8 *crypt_freebytes(__u8 *bytes);
/* loescht eine Folge von Bytes aus dem Speicher */

/* Prozeduren -------------------------------------- */

__u16 *crypt_generatekey(__u32 n,random_seed *seed)
{
  __u16 *key;
  __u32 i;
  if (seed==NULL)
    return NULL;
  key=(__u16*)malloc(n*sizeof(__u16));
  if (key==NULL)
    return NULL;
  for (i=0;i<n;i++)
    key[i]=random_random(seed);
  return key;
}

__u16 *crypt_freekey(__u16 *key)
{
  if (key!=NULL)
    free((void*)key);
  return NULL;
}

__u8 *crypt_generatebytes(__u32 n,random_seed *seed)
{
  __u32 i;
  __u8 *bytes;
  if (seed==NULL)
    return NULL;
  bytes=(__u8*)malloc(n*sizeof(__u8));
  if (bytes==NULL)
    return NULL;
  for (i=0;i<n;i++)
    bytes[i]=(__u8)random_random(seed);
  return bytes;
}

__u8 *crypt_convertbytes(__u32 n,__u16 *key)
{
  __u32 i;
  __u8 *bytes;
  if (key==NULL)
    return NULL;
  bytes=(__u8*)malloc(n*sizeof(__u8));
  if (bytes==NULL)
    return NULL;
  for (i=0;i<n;i++)
    bytes[i]=(__u8)((__u16)key[i]&0xFF);
  return bytes;
}

char crypt_xor(__u32 len,__u32 n,__u8 *data,__u8 *code)
{
  __u32 i;
  if (code==NULL)
    return 0;
  for (i=0;i<len;i++)
    data[i]=data[i]^code[i%n];
  return 1;
}

char crypt_shake(__u32 len,__u32 n,__u8 *data,__u16 *code,char encode)
{
  __u32 i,j,nr;
  __u8 b;
  if (code==NULL)
    return 0;
  for (i=0;i<len;i++) {
    if (encode)
      j=i;
    else
      j=len-i-1;
    nr=code[j%n]%len;
    b=data[nr];
    data[nr]=data[j];
    data[j]=b;
  }
  return 1;
}

__u8 *crypt_freebytes(__u8 *bytes)
{
  if (bytes!=NULL)
    free((void*)bytes);
  return NULL;
}

#endif
